/*
 *****************************************************
 *
 *  SaVi by Robert Thurman (thurman@geom.umn.edu) and
 *          Patrick Worfolk (worfolk@alum.mit.edu).
 *
 *  Copyright (c) 1997 by The Geometry Center.
 *  This file is part of SaVi.  SaVi is free software;
 *  you can redistribute it and/or modify it only under
 *  the terms given in the file COPYRIGHT which you should
 *  have received along with this file.  SaVi may be
 *  obtained from:
 *  http://savi.sourceforge.net/
 *  http://www.geom.uiuc.edu/locate/SaVi
 *
 *****************************************************
 *
 * globals.h
 *
 * Global variable declarations
 *
 * $Id: globals.h,v 1.7 2004/01/06 03:57:56 lloydwood Exp $
 */

#ifndef _GLOBALS_H_
#define _GLOBALS_H_

int geomview_module; /* TRUE if program is a geomview module */
int geomview_logo; /* TRUE if SaVi logo is shown on camera */

unsigned int texture_flag; /* TRUE if we're texturemapping */
unsigned int geomview_dynamic_texture; /* TRUE if we're to write scratchfile */

unsigned int coverage_large_map; /* TRUE if using large coverage map */

typedef enum {
  CYLINDRICAL=0, UNPROJECTED, SINUSOIDAL, NUM_PROJECTIONS
} coverage_projections;

int coverage_projection;

unsigned char * cyl_foreground;
unsigned char * sin_foreground;
unsigned char * unp_foreground;

/* sinusoidal projection is centered on America - rotated -90 degrees */
/* cylindrical projection is normal - 0 degrees rotation */
float Longitude_Center_Line;

char * first_filename;

extern char EMPTY_str[];  /* empty string */
char *Version; /* string containing who and when compiled */

int motion;   /* TRUE to move satellites, FALSE to stop them */
int reset;    /* TRUE to reset satellites to original positions */

int single_step; /* TRUE if take only one step */
double delta_t; /* time increment */
double coverage_angle; /* angle for footprints */
double tracks_interval; /* time interval for computing ground tracks */
unsigned int transforms_needed; /* flag for sending transforms to gv */

/* size of image for coverage display */
unsigned int Image_Width;
unsigned int Image_Height;

/* size of fisheye */
unsigned int Fisheye_Diameter;

/* coverage colors */
int DIV0, DIV1, DIV2, DIV3, DIV4;
int DEC0, DEC1, DEC2, DEC3, DEC4;

#endif
/* !_GLOBALS_H_ */
