#
######################################################
#
#  SaVi by Robert Thurman (thurman@geom.umn.edu) and
#          Patrick Worfolk (worfolk@alum.mit.edu).
#
#  Copyright (c) 1997 by The Geometry Center.
#  This file is part of SaVi.  SaVi is free software;
#  you can redistribute it and/or modify it only under
#  the terms given in the file COPYRIGHT which you should
#  have received along with this file.  SaVi may be
#  obtained from:
#  http://savi.sourceforge.net/
#  http://www.geom.uiuc.edu/locate/SaVi
#
######################################################
#
# coverage.tcl
#
# $Id: coverage.tcl,v 1.16 2004/07/16 14:37:39 lloydwood Exp $

# Note: IM_W and IM_H are linked to C variables for the image size


set im_x_offset 1
set im_y_offset 1

proc coverage(build) {} {
    global FONT COLOR COLDIV COLDEC coverage_flag no_access_flag tracks_flag \
	coverage_proj_flag coverage_angle_flag project_tracks_flag \
	coverage_projection coverage_angle IM_W IM_H im_x_offset im_y_offset \
	coverage_flag_dismissed tracks_flag_dismissed no_access_flag_dismissed


    if [winfo exists .coverage] {
	wm deiconify .coverage
	raise .coverage
	return
    }

    set name [build_Toplevel coverage]

    wm protocol $name WM_DELETE_WINDOW coverage(dismiss)

    build_Title $name "View of global map and ground coverage"

    set cmd [build_CmdFrame $name cmd]

    set c "$cmd.c"
    # canvas counts from 0
    canvas $c -bg black -width $IM_W -height $IM_H
    pack $c -fill x -side top
    set im [image create photo im_coverage -width [expr $IM_W*2] -height $IM_H]
    $c create image [expr $IM_W+$im_x_offset] [expr $IM_H/2+$im_y_offset] \
	-image $im

    # catch button click in map to get fisheye Lat/Long
    bind $c <Button-1> {coverage(lat_long) %x %y}

    set cmdq [build_StdFrame $cmd q]
    build_IPopupMenu $cmdq p0 "Map projection used is:" \
        coverage_projection coverage(map) {"cylindrical" "unprojected" "sinusoidal"}
    pack $cmdq.p0 -side left

    build_LabelEntryColumns $cmdq p1 \
	{text "" {"Last loaded:"}} \
	{label "" {last_filename}}
    pack $cmdq.p1
    pack $cmdq -anchor w

    set cmda [build_StdFrame $cmd a]
    build_Optionslist $cmda olist \
	{"Show coverage" coverage_flag} \
	{"Add interval decay" no_access_flag} \
	{"Show satellites" tracks_flag}
# this will give a numerical idea of coverage by latitude
#       {"Print access data" print_access_flag}
# this is hacked into utils.tcl instead.
#	{"Record ground tracks    and project forwards" project_tracks_flag}
    pack $cmda.olist -side left

    build_StdFrame $cmda ce
    # the background colors should match the colors in coverage_vis.c
    build_StdFrame $cmda.ce cf
    label $cmda.ce.cf.0 -text "diversity of coverage now:" \
	    -font $FONT(label)
    button $cmda.ce.cf.1 -text "1" -font $FONT(fixed) -bg $COLDIV(1) \
	    -activebackground $COLDIV(1) -command "coverage(diversitycolor) 1" \
	    -relief groove
    button $cmda.ce.cf.2 -text "2" -font $FONT(fixed) -bg $COLDIV(2) \
	    -activebackground $COLDIV(2) -command "coverage(diversitycolor) 2" \
	    -relief groove
    button $cmda.ce.cf.3 -text "3" -font $FONT(fixed) -bg $COLDIV(3) \
	    -activebackground $COLDIV(3) -command "coverage(diversitycolor) 3" \
	    -relief groove
    button $cmda.ce.cf.4 -text "4+" -font $FONT(fixed) -bg $COLDIV(4) \
	    -activebackground $COLDIV(4) -command "coverage(diversitycolor) 4" \
	    -relief groove
    label $cmda.ce.cf.5 -text "satellites" -font $FONT(label)
    pack $cmda.ce.cf.0 $cmda.ce.cf.1 $cmda.ce.cf.2 $cmda.ce.cf.3 $cmda.ce.cf.4 \
         $cmda.ce.cf.5 -expand 1 -fill x -side left -ipadx 2m -ipady 2m \
	 -padx 1m -pady 1m
    pack $cmda.ce.cf -side top

    build_StdFrame $cmda.ce cg
    label $cmda.ce.cg.0 -text "age of decayed coverage:" -font $FONT(label)
    button $cmda.ce.cg.1 -text "1" -font $FONT(fixed) -bg $COLDEC(1) \
	    -activebackground $COLDEC(1) -command "coverage(decaycolor) 1" \
	    -relief groove
    button $cmda.ce.cg.2 -text "2" -font $FONT(fixed) -bg $COLDEC(2) \
	    -activebackground $COLDEC(2) -command "coverage(decaycolor) 2" \
	    -relief groove
    button $cmda.ce.cg.3 -text "3" -font $FONT(fixed) -bg $COLDEC(3) \
	    -activebackground $COLDEC(3) -command "coverage(decaycolor) 3" \
	    -relief groove
    button $cmda.ce.cg.4 -text "4+" -font $FONT(fixed) -bg $COLDEC(4) \
	    -activebackground $COLDEC(4) -command "coverage(decaycolor) 4" \
	    -relief groove
    label $cmda.ce.cg.5 -text "intervals" -font $FONT(label)
    pack $cmda.ce.cg.0 $cmda.ce.cg.1 $cmda.ce.cg.2 $cmda.ce.cg.3 $cmda.ce.cg.4 \
         $cmda.ce.cg.5 -expand 1 -fill x -side left -ipadx 2m -ipady 2m \
	 -padx 1m -pady 1m
    pack $cmda.ce.cg -side top

    pack $cmda.ce -side right -fill both

    pack $cmda -fill both

    build_LabelEntryColumns $cmd le2 \
	{lcheckbox "" project_tracks_flag} \
	{ientry "" {tracks_interval}} \
	{text "" {hours}}

    build_IPopupMenu $cmd p2 "Coverage angle describes:" \
	coverage_angle_flag coverage(foot) {"ground terminal mask elevation" "cones from satellites"}

    pack $cmd.p2 -fill both -expand 1

    build_LabelEntryColumns $cmd le1 \
	{text "" "angle:"} \
	{dentry "" {coverage_angle}} \
	{label "" {coverage}} \
	{label "" {mean_coverage}}

    pack $cmd.le1

    build_LabelEntryColumns $cmd le0 \
	{label "" {coverage_init_time}} \
	{label "" {coverage_time}} \
	{text "" "Time:"} \
	{label "" {time}}

    pack $cmd.le0 -fill both -side bottom -expand 1
    pack $cmd -fill both -side top -expand 1

    # an explicit Dismiss Window button halts computation.
    build_Buttonbar $name dbbar \
	{"Refresh" coverage(foot)} \
	{"Random colors" coverage(random_colors)} \
	{"Restore colors" coverage(restore_colors)} \
        {"Save coverage" coverage(save_map)} \
	{"Dismiss window" coverage(dismiss)}

    set coverage_flag $coverage_flag_dismissed
    set tracks_flag $tracks_flag_dismissed

    coverage(update)
}

proc coverage(update_colors) {} {
global DIV1 DIV2 DIV3 DIV4 DEC1 DEC2 DEC3 DEC4 COLDIV COLDEC

    for {set i 1} {$i < 5} {incr i} {
        set name DIV$i
	set COLDIV($i) [eval format "#%06x" $$name]
	.coverage.cmd.a.ce.cf.$i configure -bg $COLDIV($i) \
	  -activebackground $COLDIV($i)

	set name DEC$i
	set COLDEC($i) [eval format "#%06x" $$name]
	.coverage.cmd.a.ce.cg.$i configure -bg $COLDEC($i) \
	  -activebackground $COLDEC($i)
    }
    coverage(update)
}

proc coverage(random_colors) {} {
    global DIV1 DIV2 DIV3 DIV4 DEC1 DEC2 DEC3 DEC4

    for {set i 1} {$i < 5} {incr i} {
	set DIV$i [random_color]
	set DEC$i [random_color]
    }
    coverage(update_colors)
}

proc coverage(restore_colors) {} {
    global COLDIV COLDEC

    coverage_color_init

    coverage(update_colors)
}

proc coverage(diversitycolor) {n} {
    global COLDIV DIV1 DIV2 DIV3 DIV4

    if {$n == "1"} {
	set s ""
    } else {
	set s "s"
    }

    set color [tk_chooseColor -initialcolor $COLDIV($n) -parent .coverage \
	    -title "SaVi: diversity color for $n satellite$s visible"]
    if {($color != "") && ($color != $COLDIV($n))} {
	set COLDIV($n) $color
	.coverage.cmd.a.ce.cf.$n configure -bg $COLDIV($n) \
		-activebackground $COLDIV($n)
	set value [string range $COLDIV($n) 1 6]
	set DIV$n [expr 0x$value]
    }
    coverage(update)
}

proc coverage(decaycolor) {n} {
    global COLDEC DEC1 DEC2 DEC3 DEC4

    if {$n == "1"} {
	set s ""
    } else {
	set s "s"
    }

    set color [tk_chooseColor -initialcolor $COLDEC($n) -parent .coverage \
	    -title "SaVi: decay color for $n interval$s since covered"]
    if {($color != "") && ($color != $COLDEC($n))} {
	set COLDEC($n) $color
	.coverage.cmd.a.ce.cg.$n configure -bg $COLDEC($n) \
		-activebackground $COLDEC($n)
	set value [string range $COLDEC($n) 1 6]
	set DEC$n [expr 0x$value]
    }
    coverage(update)
}

proc coverage(save_map) {} {
    global Save last_map_filename

    if ![winfo exists .coverage] {
	coverage(build)
    }

    set types {
        {"SaVi coverage map" {.ppm} }
    }

    set filename [tk_getSaveFile -filetypes $types \
	          -parent .coverage -title "SaVi: save coverage map" ]

    if {$filename == ""} return

    satellites COVERAGE_SAVE_MAP $filename

    set last_map_filename $filename
}

proc coverage(dismiss) {} {
    global coverage_flag tracks_flag \
	   coverage_flag_dismissed tracks_flag_dismissed

    set coverage_flag_dismissed $coverage_flag
    set tracks_flag_dismissed $tracks_flag

    # turn off flags to avoid computing drawing when window not shown
    set coverage_flag 0
    set tracks_flag 0

    destroy .coverage
}

proc coverage(update) {} {
    global coverage_flag no_access_flag tracks_flag

    if {![winfo exists .coverage]} return

    set flag $no_access_flag
    set no_access_flag 1
    if {$flag == 0} {
	set no_access_flag 0
    }

    set flag $coverage_flag
    set coverage_flag 1
    if {$flag == 0} {
	set coverage_flag 0
    }

    set flag $tracks_flag
    set tracks_flag 1
    if {$flag == 0} {
	set tracks_flag 0
    }

    update
}

proc coverage(lat_long) {x y} {
    global Fisheye coverage_projection IM_W IM_H im_x_offset im_y_offset
    global Longitude_Center_Line

    set x [expr $x-$im_x_offset]
    set y [expr $y-$im_y_offset]

    set HALF_X [expr $IM_W/2]
    set HALF_Y [expr $IM_H/2]


    if {$coverage_projection == 0} {
	# cylindrical
	set long [expr (($x%$IM_W)-$HALF_X)/double($IM_W)*360.0+$Longitude_Center_Line]
	set y [expr 1-$y/double($IM_H)*2]
	if {$y < -1} {
	    set y -1
	} elseif {$y > 1} {
	    set y 1
	}
	set lat [expr asin($y)/3.1416*180.0]
    } elseif {$coverage_projection == 1} {
	# unprojected
	set long [expr (($x%$IM_W)-$HALF_X)/double($IM_W)*360.0+$Longitude_Center_Line]
	set y [expr 1-$y/double($IM_H)*2]
	if {$y < -1} {
	    set y -1
	} elseif {$y > 1} {
	    set y 1
	}
	set lat [expr $y*90]
    } elseif {$coverage_projection == 2} {
	# sinusoidal
	set lat [expr ($HALF_Y-$y)/double($HALF_Y)*90.0]
	if {abs($lat) == 90} {
	    set long 0
	} else {
	    set long [expr \
		    (($x%$IM_W)-$HALF_X)/double($HALF_X)/cos($lat/180.0*3.14159)*180.0+$Longitude_Center_Line]
	}
	# below presumes that LCR is -90
	if {$long > 90} {
	    set long 90
	} elseif {$long < -270} {
	    set long -270
	}
    }

    if {$long <= -180} {
	set long [expr $long+360.0]
    }

    set Fisheye(lat_degrees) [expr {$lat<0 ? -int(-$lat) : int($lat)} ]
    set Fisheye(lat_minutes) [expr abs($lat-$Fisheye(lat_degrees))*60.0]
    set Fisheye(long_degrees) [expr {$long<0 ? -int(-$long) : int($long)} ]
    set Fisheye(long_minutes) [expr abs($long-$Fisheye(long_degrees))*60.0]

    # We could call fisheye(apply) here to set the coordinates directly,
    # but it's more useful to just know where you've clicked without also
    # changing the fisheye location.

}

proc coverage(foot) {} {
    coverage(update)
}

proc coverage(map) {} {
    global geomview_module coverage_proj_flag coverage_projection texture_flag

    set coverage_proj_flag [expr 1 - $coverage_proj_flag]

    if {$geomview_module && ($coverage_projection != 0) && ($texture_flag == 1)} {
	set texture_flag 0
	set texture_flag 1
    }

    coverage(update)
}
